
/**
 * @overview Defines the Stout session middleware.
 *
 * @module stout-session/middleware/SessionMiddleware
 */

(function() {
  var Middleware, Session, SessionMiddleware, defaults,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  defaults = require('lodash/defaults');

  Middleware = require('stout-core/middleware/Middleware');

  Session = require('../model/Session');


  /**
   * Session middleware responsible for handling session cookies.
   *
   * @param {stout-session/provider/SessionProvider} provider - The session
   * provider.
   *
   * @param {object} [options] - Middleware options.
   *
   * @param {string} [options.cookie] - The session cookie name.
   *
   * @param {string} [options.key] - The session cookie signing key. If none is
   * set the session cookie is not signed.
   *
   * @exports stout-session/middleware/SessionMiddleware
   * @extends stout-core/middleware/Middleware
   * @constructor
   */

  module.exports = SessionMiddleware = (function(superClass) {
    extend(SessionMiddleware, superClass);

    function SessionMiddleware(__provider, options) {
      this.__provider = __provider;
      SessionMiddleware.__super__.constructor.call(this);
      this.__options = defaults(options, {
        cookie: 'sid',
        key: null
      });
    }


    /**
     * Retrieves the session if one already exists, or creates a new session for
     * the current client. If a session already exists, it's expiry time is updated
     * and re-saved to the database. The `session` member is set on the request to
     * the existing (or created) session object.
     *
     * @param {stout-server/http/Request} req - The HTTP request.
     *
     * @param {stout-server/http/Response} res - The HTTP response.
     *
     * @param {function} next - The middleware callback function which is called
     * when the middleware has finished it's tasks.
     *
     * @member fn
     * @memberof stout-session/middleware/SessionMiddleware
     */

    SessionMiddleware.prototype.fn = function(req, res, next) {
      var createNewSession, handleErr, setSession, sid;
      sid = req.cookie[this.__options.cookie];
      handleErr = function(e) {
        return next(e);
      };
      setSession = (function(_this) {
        return function(session) {
          var attrs;
          req.session = session;
          attrs = {
            expires: session.expires,
            httpOnly: true
          };
          if (_this.__options.key) {
            attrs.signed = _this.__options.key;
          }
          res.setCookie(_this.__options.cookie, session.key, attrs);
          res.on('send', function() {
            if (session.$dirty) {
              return session.save();
            }
          });
          return next();
        };
      })(this);
      createNewSession = (function(_this) {
        return function() {
          var session;
          session = _this.__provider.create();
          return session.save().then(function() {
            return setSession(session);
          })["catch"](handleErr);
        };
      })(this);
      if (sid) {
        this.__provider.get(sid).then(function(session) {
          if (session && !session.expired) {
            return session.touch().then(function() {
              return setSession(session);
            });
          } else {
            return createNewSession();
          }
        })["catch"](handleErr);
      } else {
        createNewSession();
      }
    };

    return SessionMiddleware;

  })(Middleware);

}).call(this);

//# sourceMappingURL=SessionMiddleware.js.map
