
/**
 * @overview Defines the user session model.
 *
 * @module stout-session/model/Session
 */

(function() {
  var Model, Session, get, getRefreshTimestamp, has, set;

  get = require('lodash/get');

  has = require('lodash/has');

  Model = require('stout-orm/model/Model');

  set = require('lodash/set');


  /**
   * Gets a refreshed session timestamp, `Date.now() + Session.maxAge`.
   *
   * @returns {number} The refresh timestamp.
   *
   * @function getRefreshTimestamp
   */

  getRefreshTimestamp = function() {
    return Date.now() + Session.maxAge * 1000;
  };


  /**
   * The Session model represents a single session object which has session data
   * and is set to expire at some time. After the session expires, it should not
   * be used, and the session data is lost.
   *
   * @exports stout-session/model/Session
   * @extends stout-orm/model/Model
   * @constructor
   */

  module.exports = Session = Model.define('Session', {

    /**
     * Each session is associated with exactly one user.
     *
     * @member {stout-orm/model/Model} user
     * @memberof stout-session/model/Session#
     */
    hasOne: 'User',
    properties: {

      /**
       * The underlying session data.
       *
       * @member {object} data
       * @memberof stout-session/model/Session#
       */
      data: {
        "default": {},
        type: 'object'
      },

      /**
       * Boolean flag indicating `true` if this session is expired.
       *
       * @member {boolean} expired
       * @memberof stout-session/model/Session#
       */
      expired: {
        get: function() {
          return this.expires.getTime() < Date.now();
        },
        enumerable: false
      },

      /**
       * The date-time when the session should expire. When getting this property
       * an instance of `Date` is returned.
       *
       * @member {number} expires
       * @memberof stout-session/model/Session#
       */
      expires: {
        "default": function() {
          return getRefreshTimestamp();
        },
        get: function(timestamp) {
          return new Date(timestamp);
        },
        set: function(dt) {
          if (dt instanceof Date) {
            return dt.getTime();
          } else {
            return dt;
          }
        },
        type: ['number', Date]
      }
    },

    /**
     * Get session data at the specified path.
     *
     * @param {string} path - The session data path (i.e. `a.b.c`)
     *
     * @returns {mixed} The session data, or `undefined`.
     *
     * @method get
     * @memberof stout-session/model/Session#
     */
    get: function(path) {
      return get(this.data, path);
    },

    /**
     * Tests if session data exists at the specified path.
     *
     * @param {string} path - The session data path (i.e. `a.b.c`)
     *
     * @returns {mixed} `true` if session data exists at the specified path,
     * otherwise `false`.
     *
     * @method has
     * @memberof stout-session/model/Session#
     */
    has: function(path) {
      return has(this.data, path);
    },

    /**
     * Refreshes the session expiration to
     *
     * @method refresh
     * @memberof stout-session/model/Session#
     */
    refresh: function() {
      return this.expires = getRefreshTimestamp();
    },

    /**
     * Sets session data at the specified path.
     *
     * @param {string} path - The session data path (i.e. `a.b.c`)
     *
     * @param {mixed} value - The session data to set.
     *
     * @method set
     * @memberof stout-session/model/Session#
     */
    set: function(path, value) {
      return set(this.data, path, value);
    },

    /**
     * Pushes the session expiration to `Date.now() + Session.maxAge` and saves the
     * session.
     *
     * @method touch
     * @memberof stout-session/model/Session#
     */
    touch: function() {
      this.refresh();
      return this.save();
    }
  });


  /**
   * The default session max-age in seconds. This value may be overridden by
   * setting the `session.maxAge` config.
   *
   * @member {string} maxAge
   * @memberof stout-session/model/Session
   */

  Session.maxAge = 60;

}).call(this);

//# sourceMappingURL=Session.js.map
