###*
# @overview Defines the Stout session middleware.
#
# @module stout-session/middleware/SessionMiddleware
###
defaults   = require 'lodash/defaults'
Middleware = require 'stout-core/middleware/Middleware'
Session    = require '../model/Session'



###*
# Session middleware responsible for handling session cookies.
#
# @param {stout-session/provider/SessionProvider} provider - The session
# provider.
#
# @param {object} [options] - Middleware options.
#
# @param {string} [options.cookie] - The session cookie name.
#
# @param {string} [options.key] - The session cookie signing key. If none is
# set the session cookie is not signed.
#
# @exports stout-session/middleware/SessionMiddleware
# @extends stout-core/middleware/Middleware
# @constructor
###
module.exports = class SessionMiddleware extends Middleware

  constructor: (@__provider, options) ->
    super()
    @__options = defaults options,
      cookie: 'sid'
      key: null


  ###*
  # Retrieves the session if one already exists, or creates a new session for
  # the current client. If a session already exists, it's expiry time is updated
  # and re-saved to the database. The `session` member is set on the request to
  # the existing (or created) session object.
  #
  # @param {stout-server/http/Request} req - The HTTP request.
  #
  # @param {stout-server/http/Response} res - The HTTP response.
  #
  # @param {function} next - The middleware callback function which is called
  # when the middleware has finished it's tasks.
  #
  # @member fn
  # @memberof stout-session/middleware/SessionMiddleware
  ###
  fn: (req, res, next) ->
    sid = req.cookie[@__options.cookie]

    # Calls the `next` function passing the error.
    handleErr = (e) -> next(e)

    # Sets the session request property, response session cookie,
    # and calls the next middleware.
    setSession = (session) =>
      req.session = session
      attrs =
        expires: session.expires
        httpOnly: true
      if @__options.key then attrs.signed = @__options.key
      res.setCookie @__options.cookie, session.key, attrs
      res.on 'send', -> if session.$dirty then session.save()
      next()

    # Creates and saves a new session.
    createNewSession = =>
      session = @__provider.create()
      session.save()
      .then -> setSession(session)
      .catch handleErr

    # Get the existing session, or create a new one.
    if sid
      @__provider.get(sid).then (session) ->
        if session and not session.expired
          session.touch().then -> setSession(session)
        else
          createNewSession()
      .catch handleErr
    else
      createNewSession()

    return
