assert            = require('chai').assert
Datastore         = require 'stout-memory-datastore/Datastore'
Observable        = require 'stout-core/event/Observable'
Promise           = require 'stout-core/promise/Promise'
Session           = require '../../../model/Session'
SessionMiddleware = require '../../../middleware/SessionMiddleware'
SessionProvider   = require '../../../provider/SessionProvider'
sinon             = require 'sinon'


class MockResponse extends Observable
  constructor: ->
    super 'send'
    @setCookie = sinon.stub()

class MockRequest
  cookie: {}



describe 'middleware/SessionMiddleware', ->
  middleware = provider = req = res = null

  beforeEach ->
    datastore = new Datastore
    Session.hasOne = []
    model = Session
    provider = new SessionProvider({datastore, model})
    middleware = new SessionMiddleware provider
    req = new MockRequest
    res = new MockResponse

  it 'creates a new session if none exists', (done) ->
    middleware.fn req, res, ->
      assert.isDefined req.session
      assert.isTrue res.setCookie.calledOnce
      provider.exists(req.session.key).then (exists) ->
        assert.isTrue exists
        done()
    return

  it 'sets the response cookie to the session ID for new sessions', (done) ->
    middleware.fn req, res, ->
      sessionKey = req.session.key
      assert.isString sessionKey
      assert.isAbove sessionKey.length, 10
      assert.isTrue res.setCookie.calledWith 'sid', sessionKey
      done()
    return

  it 'retrieves the existing session using the request cookie', (done) ->
    session = new Session
    req.cookie.sid = session.key
    provider.put(session).then ->
      middleware.fn req, res, ->
        assert.equal req.session.key, session.key
        done()
    return

  it 'creates a new session if the existing session is expired', (done) ->
    session = new Session
    session.expires = Date.now() - 1000
    req.cookie.sid = session.key
    provider.put(session).then ->
      middleware.fn req, res, ->
        assert.notEqual req.session.key, session.key
        done()
    return

  it 'refreshes the existing session\'s expiration time', (done) ->
    session = new Session
    session.expires = Date.now() + 1000
    req.cookie.sid = session.key
    provider.put(session).then ->
      middleware.fn req, res, ->
        assert.isAbove req.session.expires.getTime(), Date.now() + 2000
        done()
    return

  it 'sets the session cookie expiration to match the session', (done) ->
    middleware.fn req, res, ->
      attrs = res.setCookie.getCall(0).args[2]
      assert.equal req.session.expires.getTime(), attrs.expires.getTime()
      done()
    return

  it 'sets the session cookie to HTTP only', (done) ->
    middleware.fn req, res, ->
      attrs = res.setCookie.getCall(0).args[2]
      assert.isTrue attrs.httpOnly
      done()
    return

  it 'saves a dirty session when the response is set', (done) ->
    middleware.fn req, res, ->
      req.session.set 'x.y.z', 123
      res.fire 'send'
      setTimeout ->
        provider.get(req.session.key).then (session) ->
          assert.equal session.get('x.y.z', 123)
          done()
      , 50
    return

  it 'calls next() with error for provider#create() error', (done) ->
    provider.create = -> throw new Error 'Test error.'
    middleware.fn req, res, (e) ->
      assert.instanceOf e, Error
      done()
    return

  it 'calls next() with error for response error', (done) ->
    res.on = -> throw new Error 'Test error.'
    middleware.fn req, res, (e) ->
      assert.instanceOf e, Error
      done()
    return

  it 'calls next() with error for rejected provider#get()', (done) ->
    provider.get = -> Promise.rejected(new Error 'Test error.')
    middleware.fn req, res, (e) ->
      assert.instanceOf e, Error
      done()
    return

  it 'calls next() with error for rejected session#touch()', (done) ->
    req.session = new Session
    req.session.touch = -> Promise.rejected(new Error 'Test error.')
    middleware.fn req, res, (e) ->
      assert.instanceOf e, Error
      done()
    return

  it 'calls next() with error for rejected session#save()', (done) ->
    req.session = new Session
    req.session.save = -> Promise.rejected(new Error 'Test error.')
    middleware.fn req, res, (e) ->
      assert.instanceOf e, Error
      done()
    return
